@@ -1738,7 +1738,7 @@ do_get_volume_free_space (GnomeVFSMethod    *method,
 	caps = ovu_caps_parser_parse (caps_str, len, &gerror);
 
 	memory = ovu_caps_get_memory_type (caps, mem_type);
-	if (memory != NULL) {
+	if (memory != NULL && ovu_caps_memory_has_free (memory)) {
 		*free_space = ovu_caps_memory_get_free (memory);
 	} else {
 		result = GNOME_VFS_ERROR_NOT_SUPPORTED;
