@@ -347,6 +347,39 @@ om_utils_check_bda (const gchar *bda)
 	return TRUE;
 }
 
+/* Swiped from libsoup, which is licensed under the LGPL
+ *   Copyright (C) 2005, Novell, Inc.
+ */
+static time_t
+soup_mktime_utc (struct tm *tm)
+{
+#if HAVE_TIMEGM
+	return timegm (tm);
+#else
+	time_t tt;
+	static const int days_before[] = {
+		0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
+	};
+
+	/* We check the month because (a) if we don't, the
+	 * days_before[] part below may access random memory, and (b)
+	 * soup_date_parse() doesn't check the return value of
+	 * parse_month(). The caller is responsible for ensuring the
+	 * sanity of everything else.
+	 */
+	if (tm->tm_mon < 0 || tm->tm_mon > 11)
+		return (time_t)-1;
+
+	tt = (tm->tm_year - 70) * 365;
+	tt += (tm->tm_year - 68) / 4;
+	tt += days_before[tm->tm_mon] + tm->tm_mday - 1;
+	if (tm->tm_year % 4 == 0 && tm->tm_mon < 2)
+		tt--;
+	tt = ((((tt * 24) + tm->tm_hour) * 60) + tm->tm_min) * 60 + tm->tm_sec;
+
+	return tt;
+#endif
+}
 
 /* According to the OBEX Spec 1.2 (page 16) the timezone part of the date
  * could be either nothing for local time or Z for utc
@@ -357,8 +390,6 @@ om_utils_parse_iso8601 (const gchar *str)
 	struct tm tm;
 	gint      nr;
 	gchar     tz;
-	time_t    time;
-	time_t    tz_offset = 0;
 
 	memset (&tm, 0, sizeof (struct tm));
 
@@ -377,24 +408,10 @@ om_utils_parse_iso8601 (const gchar *str)
 		return -1;
 	}
 
-	time = mktime (&tm);
-
-#if defined(HAVE_TM_GMTOFF)
-	tz_offset = tm.tm_gmtoff;
-#elif defined(HAVE_TIMEZONE)
-	tz_offset = -timezone;
-	if (tm.tm_isdst > 0) {
-		tz_offset += 3600;
+	if (nr == 7) {
+		return soup_mktime_utc (&tm);
+	} else {
+		return mktime (&tm);
 	}
-#endif
-
-	if (nr == 7) { /* Date/Time was in localtime (to remote device)
-			* already. Since we don't know anything about the
-			* timezone on that one we won't try to apply UTC offset
-			*/
-		time += tz_offset;
-	}
-
-	return time;
 }
 
